@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl 4608 2008-12-11 13:51:32Z sysjkf $
+#   $Id: ConfigDefs.pl 4972 2009-12-04 14:39:05Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -50,6 +50,14 @@ __DATA__
 
 [Translation,Translation]
 
+AFilenameRules                  = ArchivesFilenameRules
+AFiletypeRules                  = ArchivesFiletypeRules
+aallowfilenames                 = ArchivesAllowFilenames
+adenyfilemimetypes              = ArchivesDenyFileMIMETypes
+adenyfiletypes                  = ArchivesDenyFiletypes
+aallowfilemimetypes             = ArchivesAllowFileMIMETypes
+aallowfiletypes                 = ArchivesAllowFiletypes
+adenyfilenames                  = ArchivesDenyFilenames
 addenvfrom			= AddEnvelopeFromHeader
 addenvto			= AddEnvelopeToHeader
 addmshmac			= AddWatermark
@@ -135,6 +143,7 @@ keepspamarchiveclean		= keepspamandmcparchiveclean
 lastafterbatch			= alwayslookeduplastafterbatch
 lastlookup			= alwayslookeduplast
 listsascores                    = includescoresinspamassassinreport
+logdelivery			= logdeliveryandnondelivery
 loghtmltags			= logdangeroushtmltags
 logfacility			= syslogfacility
 logsaactions			= logspamassassinruleactions
@@ -158,6 +167,7 @@ mshmacnull			= TreatInvalidWatermarksWithNoSenderAsSpam
 mshmacvalid			= WatermarkLifetime
 namemodifysubject		= filenamemodifysubject
 namesubjecttext			= filenamesubjecttext
+newheadersattop			= placenewheadersattopofmessage
 noisyviruses			= nonforgingviruses
 normalrbls                      = spamliststobespam
 nosenderprecedence		= nevernotifysendersofprecedence
@@ -169,6 +179,8 @@ phishingnumbers			= alsofindnumericphishing
 phishingsubjecttag		= phishingsubjecttext
 phishingwhitelist		= phishingsafesitesfile
 outqueuedir			= outgoingqueuedir
+procdbattempts			= maximumprocessingattempts
+procdbname			= processingattemptsdatabase
 quarantinesilent		= quarantinesilentviruses
 quarantineuser			= quarantineuser
 quarantinegroup			= quarantinegroup
@@ -206,6 +218,7 @@ mcpmodifysubject		= mcpmodifysubject
 sizemodifysubject		= sizemodifysubject
 sizesubjecttext			= sizesubjecttext
 spamassassintempdir		= spamassassintemporarydir
+spaminfected			= VirusNamesWhichAreSpam
 spammodifysubject		= spammodifysubject
 spamscorenotstars		= spamscorenumberinsteadofstars
 spamstars			= spamscore
@@ -226,9 +239,14 @@ mcplistsascores			= includescoresinmcpreport
 mcpreqspamassassinscore		= mcprequiredspamassassinscore
 mcpwhitelist			= isdefinitelynotmcp
 syntaxcheck			= automaticsyntaxcheck
+unpackole			= UnpackMicrosoftDocuments
 unscannedheader			= unscannedheadervalue
 usedefaultswithmanyrecips       = usedefaultruleswithmultiplerecipients
 tagphishingsubject		= phishingmodifysubject
+unzipmaxmembers			= UnzipMaximumFilesPerArchive
+unzipmaxsize			= UnzipMaximumFileSize
+unzipmembers			= UnzipFilenames
+unzipmimetype			= UnzipMimeType
 #virusbeforespammcp		= virusscanningbeforespamormcp
 virusmodifysubject		= virusmodifysubject
 virusscan			= virusscanning
@@ -262,7 +280,7 @@ clamavspam		0	no	0	yes	1
 debug			0	no	0	yes	1
 debugspamassassin	0	no	0	yes	1
 deliverinbackground	1	no	0	yes	1
-getipfromheader		0	no	0	yes	1
+logdelivery		0	no	0	yes	1
 lognonspam		0	no	0	yes	1
 logsaactions		0	no	0	yes	1
 logsilentviruses	0	no	0	yes	1
@@ -318,7 +336,7 @@ MaxDirtyMessages		30
 MaxGSSize			20000
 MaxGSTimeouts			10
 MaxSpamAssassinTimeouts		10
-MaxSpamCheckSize		150000
+ProcDBAttempts			6
 QueueScanInterval		6
 RBLTimeoutLen			10
 RestartEvery			14400
@@ -344,6 +362,7 @@ ClamWatchFiles		/usr/local/share/clamav/*.cvd
 CustomFunctionsDir	/usr/lib/MailScanner/MailScanner/CustomFunctions
 FileCommand		/usr/bin/file
 FirstCheck		mcp
+getipfromheader		0
 GunzipCommand		/bin/gunzip
 inqueuedir		/var/spool/mqueue.in
 LDAPbase
@@ -357,6 +376,7 @@ MailScannerVersionNumber	1.0.0
 MaxSpamAssassinSize		30000
 MinimumCodeStatus	supported
 MTA			sendmail
+ProcDBName		/var/spool/MailScanner/incoming/Processing.db
 QuarantineUser
 QuarantineGroup
 QuarantinePerms		0600
@@ -374,6 +394,7 @@ SpamAssassinLocalRulesDir
 SpamAssassinLocalStateDir	
 SpamAssassinDefaultRulesDir	
 SpamAssassinInstallPrefix	
+SpamInfected		Sane*UNOFFICIAL
 SpamStarsCharacter	s
 MCPMaxSpamAssassinSize		100000
 MCPSpamAssassinUserStateDir
@@ -428,6 +449,7 @@ LookForUU		0	no	0	yes	1
 MultipleHeaders		append	append	append	replace	replace	add	add
 NameModifySubject	start	no	0	yes	1	start	start	end	end
 NoticeFullHeaders	1	no	0	yes	1
+RejectMessage		0	no	0	yes	1
 ScannedModifySubject	0	no	0	yes	1	start	start	end	end
 SendNotices		1	no	0	yes	1
 SignAlreadyScanned	0	no	0	yes	1
@@ -448,6 +470,7 @@ MCPChecks		0	no	0	yes	1
 MCPDetail		1	no	0	yes	1
 MCPListSAScores		0	no	0	yes	1
 MCPWhitelist		0	no	0	yes	1
+UnpackOle		1	no	0	yes	1
 UseSACache		1	no	0	yes	1
 VirusModifySubject	start	no	0	yes	1	start	start	end	end
 warningisattachment	1	no	0	yes	1
@@ -503,6 +526,7 @@ HighSpamAssassinScore		10
 MaxAttachmentSize		-1
 MaxMessageSize			0
 MaxParts			200
+MaxSpamCheckSize		150000
 MaxSpamListTimeouts		7
 MaxZipDepth			2
 MCPErrorScore			1
@@ -513,9 +537,12 @@ MinStars			0
 mshmacvalid			604800
 NormalRBLs			1
 ReqSpamAssassinScore		6
+unzipmaxmembers			0
+unzipmaxsize			50000
 
 [First,Other]
 Antiword			/usr/bin/antiword -f
+ArchivesAre			zip rar ole
 AttachmentCharset		ISO-8859-1
 attachimageinternalname
 attachimagename
@@ -548,6 +575,7 @@ ScoreFormat			%d
 Sendmail2			/usr/sbin/sendmail
 SpamHeader			X-MailScanner-SpamCheck:
 SpamList
+SpamVirusHeader			X-MailScanner-SpamVirus-Report:
 MCPSubjectText			{MCP?}
 SpamSubjectText			{Spam?}
 SpamStarsHeader			X-MailScanner-SpamScore:
@@ -562,6 +590,8 @@ NonMCPActions		deliver
 MCPActions		deliver
 HighScoreMCPActions	deliver
 SizeSubjectText		{Size}
+unzipmembers		*.txt *.ini *.log *.csv
+unzipmimetype		text/plain
 
 [All,YesNo]
 AddEnvFrom		1	no	0	yes	1
@@ -589,12 +619,12 @@ FindPhishing		1	no	0	yes	1
 markinfectedmessages	1	no	0	yes	1
 PhishingHighlight	1	no	0	yes	1
 HtmlToText		0	no	0	yes	1
+NewHeadersAtTop		0	no	0	yes	1
 PhishingNumbers		1	no	0	yes	1
 QuarantineInfections	1	no	0	yes	1
 QuarantineModifiedBody	0	no	0	yes	1
 QuarantineSilent	0	no	0	yes	1
 QuarantineWholeMessage	0	no	0	yes	1
-RejectMessage		0	no	0	yes	1
 ReplaceTNEF		2	no	0	add	1	replace	2
 sadecodebins		0	no	0	yes	1
 ScanMail		1	no	0	yes	1
@@ -613,6 +643,14 @@ ZipAttachments		0	no	0	yes	1
 [All,Other]
 # This is the other stuff that came up in the search that I haven't
 # figured out what to do with yet...
+aallowfilenames
+adenyfilemimetypes
+adenyfiletypes
+aallowfilemimetypes
+aallowfiletypes
+adenyfilenames
+afilenamerules
+afiletyperules
 ArchiveMail
 AttachZipIgnore			.zip .rar .gz .tgz .mpg .mpe .mpeg .mp3 .rpm
 ClamdLockFile
