@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Postfix.pm 4522 2008-08-20 15:19:23Z sysjkf $
+#   $Id: Postfix.pm 4862 2009-06-15 15:45:02Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ use Encode;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4522 $, 10;
+$VERSION = substr q$Revision: 4862 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -296,8 +296,9 @@ sub new {
     my $MaxpRecPos = 0; # Max position where a p record might occur.
     my @npos = (); # save all p record positions, although we only
                    # look at/use the first and last.
-    my $ipfromheader = ''; # The IP address from the first Received: line
-    my $read1strcvd = 0; # Have we read the 1st Received: line?
+    #my $ipfromheader = ''; # The IP address from the first Received: line
+    #my $read1strcvd = 0; # Have we read the 1st Received: line?
+    my(@rcvdiplist);
 
     #print STDERR "In ReadQf\n";
     #$message->{store}->print();
@@ -359,6 +360,7 @@ sub new {
       $message->{deleted} = 1;
       $message->{gonefromdisk} = 1; # Don't try to delete the original
       $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+      #20090421 $message->{abandoned} = 1;
       return 0;
     }
 
@@ -415,11 +417,10 @@ sub new {
         $pRecordsFound = 1; # If we find a p record we need remember this to handle body ...
 	next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	if ($recdata+0 > $MaxpRecPos) {
-	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	    $ErrorFound = 1;
 	    last;
 	}
-	push @npos, $recdata+0; # save jumpto position, for loop detection.
 	if ($OriginalPos == 0) {
 	   # Jump to after E record and commence
 	   $OriginalPos = tell $RQf;
@@ -428,30 +429,29 @@ sub new {
 	} else {
 	  # We're at the return point, or moving even furtehr away...
 	  if ($recdata+0 < $OriginalPos) {
-	     # Error exit, carp or whatever. Probably should do the
-	     # nice exit Jules does above. For now, warn and use
-	     # OriginalPos
-	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	     seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	     $OriginalPos = 0; # We're home again... Reset to prepare for
-			       # any new warps
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	     #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0;
 	     @npos = ();
+	     $ErrorFound = 1;
+	     last;
 	  } else {
 	    seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	    if ($recdata+0 < $npos[$#npos-1]) {
-	       if ($recdata+0 >= $npos[0]) {
-		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+            foreach $_ (@npos) {
+              if ($_+0 eq $recdata+0) {
 		  $ErrorFound = 1;
-		  $OriginalPos = 0; # Reset to not fool next segment loop
-	          @npos = ();
 		  last;
 	       }
-	       $OriginalPos = 0; # We're home again... Reset to prepare for
-		                 # any new warps
+	    }
+            if ($ErrorFound) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $OriginalPos = 0; # Reset to not fool next segment loop
 	       @npos = ();
+		  last;
 	    }
 	  }
 	}
+	push @npos, $recdata+0; # save jumpto position, for loop detection.
       } else {
         # Some other record type. Just store it and move on.
         push @{$message->{metadata}}, "$rectype$recdata";
@@ -470,6 +470,11 @@ sub new {
         last;
       }
       last if $rectype eq 'N' && $recdata eq "";
+      if (!defined($rectype)) {
+         $ErrorFound = 1;
+      #print STDERR "RECTYPER ERROR: $rectype, $recdata\n";
+         last;
+      }
       if ($rectype eq 'p') {
 	 # //Glenn 2007-01-16
 	 # Handle p records (GOTO like in-place edit thing) by reading
@@ -484,43 +489,40 @@ sub new {
          $pRecordsFound = 1; # If we find a p record we need remember this to handle body ...
 	 next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	 if ($recdata+0 > $MaxpRecPos) {
-	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	    $ErrorFound = 1;
 	    last;
 	 }
-	 push @npos, $recdata+0; # save jumpto position, for loop detection.
 	 if ($OriginalPos == 0) {
 	    # Jump to after E record and commence
 	    $OriginalPos = tell $RQf;
 	    seek $RQf, $recdata+0, 0; # jump. we should check this works.
-#	    next;
 	 } else {
-	   # We're at the return point, or moving even furtehr away...
+	   # We're at the return point, or moving even further away...
 	   if ($recdata+0 < $OriginalPos) {
-	      # Error exit, carp or whatever. Probably should do the
-	      # nice exit Jules does above. For now, warn and use
-	      # OriginalPos
-	      MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	      seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	      $OriginalPos = 0; # We're home again... Reset to prepare for
-			        # any new warps
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	     #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0;
 	      @npos = ();
+	     $ErrorFound = 1;
+	     last;
 	   } else {
 	     seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	     if ($recdata+0 < $npos[$#npos-1]) {
-	        if ($recdata+0 >= $npos[0]) {
-        	   MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+            foreach $_ (@npos) {
+              if ($_+0 eq $recdata+0) {
 		   $ErrorFound = 1;
-	           $OriginalPos = 0; # Reset to not fool next segment loop...
-	           @npos = ();
 		   last;
 	        }
-	        $OriginalPos = 0; # We're home again... Reset to prepare for
-		                  # any new warps
+	    }
+            if ($ErrorFound) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $OriginalPos = 0; # Reset to not fool next segment loop
 	        @npos = ();
+		  last;
 	     }
 	   }
 	 }
+	 push @npos, $recdata+0; # save jumpto position, for loop detection.
 	 next; # done, don't add a spurious "converted p to N" record.
       }
       next if ($rectype eq 'w'); # Skip deleted (w) records ... Else they will be transformed to Normal (N) records.
@@ -541,28 +543,41 @@ sub new {
           $InSubject = 0;
         }
       }
-      if ($recdata =~ /^Received: .+\[(\d+\.\d+\.\d+\.\d+)\]/i) {
-        unless ($read1strcvd) {
-          $ipfromheader = $1;
-          $read1strcvd = 1;
-        }
-        unless ($IPFound) {
-          $message->{clientip} = $1;
-          $IPFound = 1;
-        }
-      } elsif ($recdata =~ /^Received: .+\[([\dabcdef.:]+)\]/i) {
-        unless ($read1strcvd) {
-          $ipfromheader = $1;
-          $read1strcvd = 1;
-        }
-        unless ($IPFound) {
-          $message->{clientip} = $1;
-          $IPFound = 1;
-        }
-      } elsif (!$IPFound &&
-               $recdata =~ /^Received: .+\(Postfix/i) {
-        $message->{clientip} = '127.0.0.1';  #spoof local sender from localhost
-        $IPFound = 1;
+      #if ($recdata =~ /^Received: .+\[(\d+\.\d+\.\d+\.\d+)\]/i) {
+      #  unless ($read1strcvd) {
+      #    $ipfromheader = $1;
+      #    $read1strcvd = 1;
+      #  }
+      #  unless ($IPFound) {
+      #    $message->{clientip} = $1;
+      #    $IPFound = 1;
+      #  }
+      #} elsif ($recdata =~ /^Received: .+\[([\dabcdef.:]+)\]/i) {
+      # Linux adds "IPv6:" on the front of the IPv6 address, so remove it
+      if ($recdata =~ /^Received:/i) {
+        my $rcvdip = '127.0.0.1';
+        if ($recdata =~ /^Received: .+?\(.*?\[(?:IPv6:)?([0-9a-f.:]+)\]/i) {
+          $rcvdip = $1;
+          #unless ($read1strcvd) {
+          #  $ipfromheader = $1;
+          #  $read1strcvd = 1;
+          #}
+          #15 if ($getipfromheader && $getipfromheader <= @rcvdiplist) {
+          #15   $message->{clientip} = $rcvdiplist[$getipfromheader-1];
+          #15   $IPFound = 1;
+          #15 }
+          #unless ($IPFound) {
+          #  $message->{clientip} = $1;
+          #  $IPFound = 1;
+          #}
+        }#15  elsif (!$IPFound &&
+         #15         $getipfromheader==1 &&
+         #15         $recdata =~ /^Received: .+\(Postfix/i) {
+         #15  $message->{clientip} = '127.0.0.1';  #spoof local sender from localhost
+         #15  $rcvdip = '127.0.0.1';
+         #15  $IPFound = 1;
+        #15 }
+        push @rcvdiplist, $rcvdip;
       }
     }
     # Must remember to add empty "X" record after the message data.
@@ -574,8 +589,8 @@ sub new {
 # Inelegant, but working. Instead of an efficient seek, we spinn through to
 # after X record. Unless we don't have a body to spin through. Also skip
 # the spin if we don't have any p records already (to not punish the normal
-# case).
-    if (!$message->{nobody} && $pRecordsFound) {
+# case). Don't spin if error found either.
+    if (!$message->{nobody} && $pRecordsFound && !$ErrorFound) {
       while(($rectype, $recdata) = ReadRecord($RQf)) {
         #print STDERR "Metadata type $rectype data \"$recdata\"\n";
         if (!defined($rectype) or $rectype eq 'X') {
@@ -595,11 +610,10 @@ sub new {
 	  # check that and act accordingly..
 	  next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	  if ($recdata+0 > $MaxpRecPos) {
-	      MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	      MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	      $ErrorFound = 1;
 	      last;
 	  }
-	  push @npos, $recdata+0; # save jumpto position, for loop detection.
 	  if ($OriginalPos == 0) {
 	     # Jump to after E record and commence
 	     $OriginalPos = tell $RQf;
@@ -608,34 +622,34 @@ sub new {
 	  } else {
 	    # We're at the return point, or moving even furtehr away...
 	    if ($recdata+0 < $OriginalPos) {
-	       # Error exit, carp or whatever. Probably should do the
-	       # nice exit Jules does above. For now, warn and use
-	       # OriginalPos
-	       MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	       seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	       $OriginalPos = 0; # We're home again... Reset to prepare for
-			         # any mew warps
+	       MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	       #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	       $OriginalPos = 0;
 	       @npos = ();
+	       $ErrorFound = 1;
+	       last;
 	    } else {
 	      seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	      if ($recdata+0 < $npos[$#npos-1]) {
-	         if ($recdata+0 >= $npos[0]) {
-		    MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+              foreach $_ (@npos) {
+                if ($_+0 eq $recdata+0) {
 		    $ErrorFound = 1;
-		    $OriginalPos = 0; # Reset to not fool next segment loop...
 		    last;
 	         }
-	         $OriginalPos = 0; # We're home again... Reset to prepare for
-		                   # any mew warps
+	      }
+              if ($ErrorFound) {
+		 MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		 $OriginalPos = 0; # Reset to not fool next segment loop
 	         @npos = ();
+		 last;
 	      }
 	    }
 	  }
+	  push @npos, $recdata+0; # save jumpto position, for loop detection.
         }
       }
       #print STDERR "\n\nErrorFound=$ErrorFound and rectype=$rectype\n\n";
       # Found errors above, means the file isn't complete... remove it from the batch and return immediately.
-      if ($ErrorFound or $rectype ne 'X') {
+      if ($ErrorFound || $rectype ne 'X') {
         #MailScanner::Log::WarnLog("No end-of-message record found in %s, " .
         #                          "retrying", $message->{id});
         # JKF 5/12/2005 This could fail with an unblessed reference error
@@ -644,6 +658,7 @@ sub new {
         $message->{deleted} = 1;
         $message->{gonefromdisk} = 1; # Don't try to delete the original
         $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+        #20090421 $message->{abandoned} = 1; # JKF 20090301 This message was scrapped
 
         return 0;
       }
@@ -710,47 +725,61 @@ sub new {
 	# safe than sorry.
 	next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	if ($recdata+0 > $MaxpRecPos) {
-	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	    $ErrorFound = 1;
 	    last;
 	}
-	push @npos, $recdata+0; # save jumpto position, for loop detection.
 	if ($OriginalPos == 0) {
 	   # Jump to after E record and commence
 	   $OriginalPos = tell $RQf;
 	   seek $RQf, $recdata+0, 0; # jump. we should check this works.
 	   next;
 	} else {
-	  # We're at the return point, or moving even furtehr away...
+	  # We're at the return point, or moving even further away...
 	  if ($recdata+0 < $OriginalPos) {
-	     # Error exit, carp or whatever. Probably should do the
-	     # nice exit Jules does above. For now, warn and use
-	     # OriginalPos
-	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	     seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	     $OriginalPos = 0; # We're home again... Reset to prepare for
-			       # any mew warps
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	     #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0;
 	     @npos = ();
+	     $ErrorFound = 1;
+	     last;
 	  } else {
 	    seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	    if ($recdata+0 < $npos[$#npos-1]) {
-	       if ($recdata+0 >= $npos[0]) {
-		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+            foreach $_ (@npos) {
+              if ($_+0 eq $recdata+0) {
 		  $ErrorFound = 1;
-		  # no need to reset $OriginalPos since this is the last segment.
 		  last;
 	       }
-	       $OriginalPos = 0; # We're home again... Reset to prepare for
-		                 # any mew warps
+	    }
+            if ($ErrorFound) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $OriginalPos = 0; # Reset to not fool next segment loop
 	       @npos = ();
+		  last;
 	    }
 	  }
 	}
+	push @npos, $recdata+0; # save jumpto position, for loop detection.
       } else {
         # Some other record type. Just store it and move on.
         push @{$message->{metadata}}, "$rectype$recdata";
       }
     }
+    #print STDERR "\n\nErrorFound=$ErrorFound and rectype=$rectype\n\n";
+    # Found errors above, means the file isn't complete... remove it from the batch and return immediately.
+    if ($ErrorFound) {
+      #MailScanner::Log::WarnLog("No end-of-message record found in %s, " .
+      #                          "retrying", $message->{id});
+      # JKF 5/12/2005 This could fail with an unblessed reference error
+      # JKF 5/12/2005 so do it by hand.
+      # JKF 5/12/2005 $message->DropFromBatch();
+      $message->{deleted} = 1;
+      $message->{gonefromdisk} = 1; # Don't try to delete the original
+      $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+      #20090421 $message->{abandoned} = 1; # JKF 20090301 This message was scrapped
+
+      return 0;
+    }
       
     # Remove all the duplicates from ->{to}
     my %uniqueto;
@@ -775,10 +804,25 @@ sub new {
 
     # If we didn't find an IP address, then put in 0.0.0.0 so that at least
     # we have something there
-    $message->{clientip} = '0.0.0.0' unless $IPFound;
+
+    # There will always be at least 1 Received: header, even if 127.0.0.1
+    push @rcvdiplist, '127.0.0.1' unless @rcvdiplist;
+    # Don't fall off the end of the list
+    $getipfromheader = @rcvdiplist if $getipfromheader>@rcvdiplist;
+    # Don't fall off the start of the list
+    $getipfromheader = 1 if $getipfromheader<1;
+    $message->{clientip} = $rcvdiplist[$getipfromheader-1];
+    $IPFound = 1;
+    #print STDERR "Using IP " . $message->{clientip} . "\n";
+
+    #$message->{clientip} = '0.0.0.0' unless $IPFound;
     # If we were told to get the IP address from the headers, and there was one
-    $message->{clientip} = $ipfromheader
-      if $getipfromheader && $read1strcvd && $ipfromheader ne "";
+    #$getipfromheader = @rcvdiplist if $getipfromheader>@rcvdiplist;
+    # If they wanted the 2nd Received from address, give'em element 1 of list
+    #$message->{clientip} = $rcvdiplist[$getipfromheader-1] if
+    #  $getipfromheader>0;
+    #$message->{clientip} = $ipfromheader
+    #  if $getipfromheader && $read1strcvd && $ipfromheader ne "";
 
     # Decode ISO subject lines into UTF8
     # Needed for UTF8 support in MailWatch 2.0
@@ -820,10 +864,13 @@ sub new {
       $message->{deleted} = 1;
       $message->{gonefromdisk} = 1; # Don't try to delete the original
       $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+      #20090421 $message->{abandoned} = 1; # Retry this message as it was ditched
 
       return 0;
     }
 
+    #20090421 $message->{abandoned} = 1 if $ErrorFound; # Mark message as scrapped
+
     return 1 if $FROMFound && $TOFound && !$ErrorFound; # && $IPFound;
     #MailScanner::Log::WarnLog("Batch: Found invalid queue file for " .
     #                          "message %s", $message->{id});
@@ -849,7 +896,7 @@ sub new {
     $len = 0;
     $shift = 0;
     while (1) {
-      read $fh, $len_byte, 1;
+      read $fh, $len_byte, 1 or return (undef, undef);
       $len_byte = ord $len_byte;
       if ($shift >= 39) {
         MailScanner::Log::WarnLog("Postfix record too long in ReadRecord()");
@@ -921,8 +968,20 @@ sub new {
 
     # Find the X record
     my($pos);
-    $pos = $#{$message->{metadata}};
-    $pos-- while ($pos >= 0 && $message->{metadata}[$pos] !~ /^X/);
+
+    # DKIM: This is for adding the header at the position of the first N record
+    # DKIM: Find the first N (or else the X as a safeguard)
+    if ($message->{newheadersattop}) {
+      $pos = 0;
+      while ($pos < $#{$message->{metadata}} &&
+             $message->{metadata}[$pos] !~ /^[NX]/) {
+        $pos++;
+      }
+    } else {
+      # DKIM: This is for adding the header at the end, pos-->X record
+      $pos = $#{$message->{metadata}};
+      $pos-- while ($pos >= 0 && $message->{metadata}[$pos] !~ /^X/);
+    }
     #print STDERR "*** AddHeader $newkey $newvalue at position $pos\n";
 
     # Need to split the new header data into the 1st line and a list of
@@ -945,14 +1004,26 @@ sub new {
   sub DeleteHeader {
     my($this, $message, $key) = @_;
 
-    my($pos);
+    my $usingregexp = ($key =~ s/^\/(.*)\/$/$1/)?1:0;
+
+    # Add a colon if they forgot it.
+    $key .= ':' unless $usingregexp || $key =~ /:$/;
+    # If it's not a regexp, then anchor it and sanitise it.
+    $key = '^' . quotemeta($key) unless $usingregexp;
+
+    my($pos, $line);
     $pos = 0;
     $pos++ while ($message->{metadata}[$pos] !~ /^M/);
     # Now points at the M record
     $pos++;
     # Now points at first N record
     while ($pos < @{$message->{metadata}}) {
-      if ($message->{metadata}[$pos] =~ /^N$key/i) {
+      $line = $message->{metadata}[$pos];
+      if ($line =~ s/^N//) {
+        unless ($line =~ /$key/i) {
+          $pos++;
+          next;
+        }
         # We have found the start of 1 occurrence of this header
         splice @{$message->{metadata}}, $pos, 1;
         # Delete continuation lines
@@ -997,7 +1068,8 @@ sub new {
   sub ReplaceHeader {
     my($this, $message, $key, $newvalue) = @_;
 
-    $this->DeleteHeader($message, $key);
+    # DKIM: Don't do DeleteHeader if adding headers at top
+    $this->DeleteHeader($message, $key) unless $message->{dkimfriendly};
     $this->AddHeader($message, $key, $newvalue);
   }
 
@@ -1649,6 +1721,7 @@ sub FindHashDirDepth {
       }
 
       $batchempty = 1;
+      my $maxattempts = MailScanner::Config::Value('procdbattempts');
 
       # Keep going until end of dir or have reached every imposed limit. This
       # now processes the files oldest first to make for fairer queue cleanups.
@@ -1692,7 +1765,12 @@ sub FindHashDirDepth {
         # If they want a particular message id, ignore it if it doesn't match
         next if $onlyid ne "" && $idtemp ne $onlyid;
 
-        my $id = $idtemp . sprintf(".%05X", int(rand 1000000)+1);
+        #my $id = $idtemp . sprintf(".%05X", int(rand 1000000)+1);
+        # Don't put a random number on the end, put a reasonable hash of
+        # the file on the end.
+        # JKF 20090423 Add a "P" in the middle of so it cannot be a number.
+        my $id = $idtemp . '.' . PostfixKey($file);
+        #print STDERR "ID = $id\n";
         my $idorig = $idtemp;
 
         #print STDERR "Adding $id to batch\n";
@@ -1700,12 +1778,74 @@ sub FindHashDirDepth {
         # Lock and read the qf file. Skip this message if the lock fails.
         $newmessage = MailScanner::Message->new($id, $queuedirname,
                                                 $getipfromheader);
-        if ($newmessage eq 'INVALID') {
+        if ($newmessage && $newmessage->{INVALID}) {
           #$invalidfiles .= "$id ";
           $invalidfiles .= "$idorig ";
+          undef $newmessage;
           next;
         }
         next unless $newmessage;
+        next if $newmessage->{INVALID};
+
+        # JKF 20090301 Skip this message if $id has been scanned
+        # too many times.
+        # JKF 20090301 Read the number of times this message id has
+        # been processed. If over the limit, then ignore it.
+        # JKF 20090301 Should ideally make it in a batch of its own,
+        # and try that once or twice, *then* skip it. But let's try
+        # a simple version first.
+        # Just do a "next" if we want to skip the message.
+        if ($maxattempts) {
+          my $nexttime = time + 120 + int(rand(240)); # 4 +- 2 minutes
+          #my $nexttime = time + 10 + int(rand(20)); # 4 +- 2 minutes
+          my @attempts = $MailScanner::ProcDBH->selectrow_array(
+                           $MailScanner::SthSelectRows, undef, $id);
+          #my @attempts = $MailScanner::ProcDBH->selectrow_array(
+          #"SELECT id,count,nexttime FROM processing WHERE (id='" . $id . "')");
+          #print STDERR "id       = \"$attempts[0]\"\n";
+          #print STDERR "count    = \"$attempts[1]\"\n";
+          #print STDERR "nexttime = \"$attempts[2]\"\n";
+          if (@attempts && $attempts[1]>=$maxattempts) {
+            MailScanner::Log::WarnLog("Warning: skipping message %s as it has been attempted too many times", $id);
+            # JKF 20090301 next;
+            # Instead of just skipping it, quarantine it and notify
+            # the local postmaster.
+            $newmessage->QuarantineDOS();
+            #print STDERR "Moving $attempts[0], $attempts[1], $attempts[2] into archive table\n";
+            $MailScanner::SthDeleteId->execute($id);
+            #$MailScanner::ProcDBH->do(
+            #  "DELETE FROM processing WHERE id='" . $id . "'");
+            $MailScanner::SthInsertArchive->execute($attempts[0], $attempts[1],
+                                               $attempts[2]);
+            #$MailScanner::ProcDBH->do(
+            #  "INSERT INTO archive (id, count, nexttime) " .
+            #  "VALUES ('$attempts[0]', $attempts[1], $attempts[2])");
+          } elsif (defined $attempts[1]) {
+            # We have tried this message before
+            if (time>=$attempts[2]) {
+              # Time for next attempt has arrived
+              $MailScanner::SthIncrementId->execute($nexttime, $id);
+              #$MailScanner::ProcDBH->do(
+              #  "UPDATE processing SET count=count+1, nexttime=$nexttime " .
+              #  " WHERE id='" . $id . "'");
+              MailScanner::Log::InfoLog(
+                "Making attempt %d at processing message %s",
+                $attempts[1]+1, $id);
+              #print STDERR "Incremented $id\n";
+            } else {
+              # Not time for next attempt yet, so ignore the message
+              $newmessage->DropFromBatch();
+              next;
+            }
+          } else {
+            # We have never seen this message before
+            $MailScanner::SthInsertProc->execute($id, 1, $nexttime);
+            #$MailScanner::ProcDBH->do(
+            #  "INSERT INTO processing (id, count, nexttime) " .
+            #  "VALUES ('" . $id . "', 1, $nexttime)");
+            #print STDERR "Inserted $id\n";
+          }
+        }
 
         $batch->{messages}{"$id"} = $newmessage;
         $newmessage->{mtime} = $mtime;
@@ -1771,6 +1911,36 @@ sub FindHashDirDepth {
     #print STDERR "Dirty stats are $DirtyMsgs msgs, $DirtyBytes bytes\n";
   }
 
+# Given a filename, read the first 256 bytes of the file and calculate a
+# checksum based on those bytes. It must consist of a string of 5 digits
+# from the range 0-9.
+sub PostfixKey {
+  my($fname) = @_;
+
+  my($fh, $data, $bytesread);
+
+  $fh = new FileHandle;
+  open($fh, "< " . $fname) or return '00000';
+  seek($fh, 0, 0) or return '00000';
+  
+  $data = '0' x 256;
+  $bytesread = read $fh, $data, 256;
+  return '00000' unless $bytesread;
+
+  # This is the "Fletcher" checksum algorithm, in case you want to look it up
+  # Thanks to jd@oddlittle.me for this one!
+  my($acc1, $acc2, $char, $checksum);
+  foreach $char (unpack "C*", $data) {
+    $acc1 = ($acc1 + $char) & 0xFF;
+    $acc2 = ($acc2 + $acc1) & 0xFF;
+  }
+  $checksum = uc(sprintf("A%04X", (($acc1<<8) + $acc2) & 0xFFFF));
+
+  close($fh);
+
+  return $checksum;
+}
+
 
 # Return the array of headers from this message, optionally with a
 # separator on the end of each one.
